//---------------------------------------------------------------------------
#include <vcl.h>
#pragma hdrstop

#include <fstream.h>
#include <Filectrl.hpp>

#include "TEFSCGalaxy.h"
#include "TEFSMain.h"
#include "TGalaxyModule.h"

//---------------------------------------------------------------------------
#pragma package(smart_init)

TEFSCGalaxy::TEFSCGalaxy() {

  GALFileName = "";
  ProjectAlias = "EFS_II";
  version = 0;
  Modified = false;

  for (int i=0; i<STAR_MAP_WIDTH; i++) {
    for (int j=0; j<STAR_MAP_HEIGHT; j++) MapTiles[i][j] = 0;
  }
  PlanetCount = 0;
  for (int i=0; i<MAX_PLANETS; i++) Planets[i] = 0;
  JumpGateCount = 0;
  for (int i=0; i<MAX_JUMP_GATES; i++) JumpGates[i] = 0;

  StructureCount = 0;

  UnitCount = 0;

}
//---------------------------------------------------------------------------

TEFSCGalaxy::~TEFSCGalaxy() {

  for (int i=0; i<MAX_PLANETS; i++) {
    if (Planets[i]) {
      delete Planets[i];
    }
  }
  GalaxyModule->PlanetTable->Active = false;
  //GalaxyModule->PlanetTable->EmptyTable();

  for (int i=0; i<MAX_JUMP_GATES; i++) {
    if (JumpGates[i]) {
      delete JumpGates[i];
    }
  }
  GalaxyModule->JumpRouteTable->Active = false;
  //GalaxyModule->JumpRouteTable->EmptyTable();

  while (Units.size() > 0) {
    delete Units.back();
    Units.pop_back();
  }
  GalaxyModule->UnitTable->Active = false;
  //GalaxyModule->UnitTable->EmptyTable();
  
  while (Structures.size() > 0) {
    delete Structures.back();
    Structures.pop_back();
  }
  GalaxyModule->StructureTable->Active = false;
  //GalaxyModule->StructureTable->EmptyTable();
  
}
//---------------------------------------------------------------------------

bool TEFSCGalaxy::OpenGalaxyFile(String fileName, const String projectAlias)
{
  EFSMain->ShowProgress(0);

  ifstream infile(fileName.c_str(), ios::binary);
  if (!infile) {
    String s = "The specified EFS Classic Galaxy file:\n"
               + fileName + "\ncould not be opened...";
    MessageDlg(s, mtInformation, TMsgDlgButtons() << mbOK, 0);
    return false;
  }

  // First, locate the appropriate set of DAT files:
  SetGALFileName(fileName);
  bool FoundDATFiles;
  if (DirectoryExists(DATPath)) {
    FoundDATFiles = true;
  }
  else {
    DATPath = GetCurrentDir();
    FoundDATFiles = SelectDirectory(DATPath, TSelectDirOpts(), 0);
    if (!DirectoryExists(DATPath)) {
      String s = "The specified EFS Classic DAT file path:\n"
                 + DATPath + "\nwas not found...";
      MessageDlg(s, mtInformation, TMsgDlgButtons() << mbOK, 0);
      return false;
    }
  }

  // Create the physical file structure for the new project database:
  ProjectAlias = projectAlias;
  GalaxyModule->CreateProject(ProjectAlias);

  // Load DAT files:
  DATPath = IncludeTrailingBackslash(DATPath);
  if (FoundDATFiles) GalaxyModule->LoadDATFiles(DATPath);

  // Determine file size:
  int pos = infile.tellg();
  infile.seekg(0L, SEEK_END);
  int filesize = infile.tellg();
  infile.seekg(0L, SEEK_SET);

  infile.read((char*)&version, sizeof(version));
  if (version < 961006) {
    String s = String("Version ") + String(version) + String(" is obsolete");
    MessageDlg(s, mtInformation, TMsgDlgButtons() << mbOK, 0);
    return false;
  }

  infile.read((char*)&UnitsInFile, sizeof(UnitsInFile));
  infile.read((char*)&MapTiles, sizeof(MapTiles));

  int index = 0;
  short section;

  String MapFileName = GalaxyModule->GetDataDirectory() + String("PlanetMaps.FS2");
  ofstream mapfile(MapFileName.c_str(), ios::binary);
  if (!mapfile) {
    String s = String("The specified EFS Planet Maps file:\n")
               + MapFileName + String("\ncould not be created...");
    MessageDlg(s, mtInformation, TMsgDlgButtons() << mbOK, 0);
    return false;
  }

  EFSMain->StatusMessage("Loading Planets");
  GalaxyModule->PlanetTable->Active = false;
  GalaxyModule->PlanetTable->EmptyTable();
  GalaxyModule->PlanetTable->Active = true;
  infile.read((char*)&section, sizeof(section));
  while (section != END_OF_SECTION) {
    String sm = String("Loading Planet: ") + String(index);
    EFSMain->StatusMessage(sm);
    Planets[index] = new TEFSCPlanet(index);
    Planets[index]->x = section;
    infile.read((char*)&Planets[index]->y, sizeof(Planets[index]->y));
    infile.read((char*)&Planets[index]->CurRotPos, sizeof(Planets[index]->CurRotPos));
    infile.read((char*)&Planets[index]->UnitPos, sizeof(Planets[index]->UnitPos));
    infile.read((char*)&Planets[index]->Name, sizeof(Planets[index]->Name));
    infile.read((char*)&Planets[index]->Owner, sizeof(Planets[index]->Owner));
    infile.read((char*)&Planets[index]->Sect, sizeof(Planets[index]->Sect));
    infile.read((char*)&Planets[index]->Flags, sizeof(Planets[index]->Flags));
    infile.read((char*)&Planets[index]->TileSetType, sizeof(Planets[index]->TileSetType));

    if (version < 961024) {
      int blah;
      infile.read((char*)&blah, sizeof(blah));
      infile.read((char*)&blah, sizeof(blah));
      infile.read((char*)&blah, sizeof(blah));
    }
    infile.read((char*)&Planets[index]->HexBuffer, sizeof(Planets[index]->HexBuffer));
    mapfile.write((char*)&Planets[index]->HexBuffer, sizeof(Planets[index]->HexBuffer));

    GalaxyModule->PlanetTable->Append();
    GalaxyModule->PlanetTableName->Value = Planets[index]->Name;
    GalaxyModule->PlanetTablePIdx->Value = index;
    GalaxyModule->PlanetTableX->Value = Planets[index]->x;
    GalaxyModule->PlanetTableY->Value = Planets[index]->y;
    GalaxyModule->PlanetTableOwner->Value = Planets[index]->Owner;
    GalaxyModule->PlanetTableSect->Value = Planets[index]->Sect;
    GalaxyModule->PlanetTableCRP->Value = Planets[index]->CurRotPos;
    GalaxyModule->PlanetTableFlags->Value = Planets[index]->Flags;
    GalaxyModule->PlanetTableTileSet->Value = Planets[index]->TileSetType;

    SetStarMapTile(index, Planets[index]->Owner, STAR_PLANET);

    index++;

    pos = infile.tellg();
    EFSMain->ShowProgress( 100 * pos / filesize );

    infile.read((char*)&section, sizeof(section));
  } // End of Planets
  PlanetCount = index;
  mapfile.close();

  // Post last record appended to the PlanetTable:
  GalaxyModule->PlanetTable->Post();
  GalaxyModule->PlanetTable->FindFirst();

  /*
  // Create dummy Planets to hold the places for any not present
  //in the GAL file(?):  (The problem with this is that EFS may
  //not know what to do with these 'dummy' Planets... OTOH, we
  //could withhold saving non-existent planets, e.g., those with
  //no name.  However, we would still need to sort the planets and
  //update all references to all planet indicies whenever a planet
  //is removed)

  for (int i=0; i<MAX_PLANETS; i++) {
    if (!Planets[i]) {
      Planets[i] = new TEFSCPlanet(i);
    }
  }

  */

  // Load in Jump Gates:
  EFSMain->StatusMessage("Loading Jump Routes");
  index = 0;
  infile.read((char*)&section, sizeof(section));
  while (section != END_OF_SECTION) {
    String sm = String("Loading Jump Route: ") + String(index);
    EFSMain->StatusMessage(sm);
    JumpGates[index] = new TEFSCJumpGate();
    JumpGates[index]->x1 = section;
    short coord;
    infile.read((char*)&coord, sizeof(coord));
    JumpGates[index]->y1 = coord;
    infile.read((char*)&coord, sizeof(coord));
    JumpGates[index]->x2 = coord;
    infile.read((char*)&coord, sizeof(coord));
    JumpGates[index]->y2 = coord;
    infile.read((char*)&JumpGates[index]->Flags, sizeof(JumpGates[index]->Flags));

    index++;

    pos = infile.tellg();
    EFSMain->ShowProgress( 100 * pos / filesize );

    infile.read((char*)&section, sizeof(section));
  } // End of Jump Gates
  JumpGateCount = index;

  UpdateJumpGates();

  GalaxyModule->JumpRouteTable->Active = false;
  GalaxyModule->JumpRouteTable->EmptyTable();
  GalaxyModule->JumpRouteTable->Active = true;
  for (int i=0; i<JumpGateCount; i++) {
    GalaxyModule->JumpRouteTable->Append();
    GalaxyModule->JumpRouteTableJRIdx->Value = i;
    GalaxyModule->JumpRouteTableP1Idx->Value = JumpGates[i]->Planet1Index;
    GalaxyModule->JumpRouteTableP2Idx->Value = JumpGates[i]->Planet2Index;
    GalaxyModule->JumpRouteTableFlags->Value = JumpGates[i]->Flags;
  }
  // Post last record appended to the JumpRouteTable:
  if (JumpGateCount > 0) {
    GalaxyModule->JumpRouteTable->Post();
    GalaxyModule->JumpRouteTable->FindFirst();
  }

  // Load in Units:
  EFSMain->StatusMessage("Loading Units");
  GalaxyModule->UnitTable->Active = false;
  GalaxyModule->UnitTable->EmptyTable();
  GalaxyModule->UnitTable->Active = true;
  index = 0;
  infile.read((char*)&section, sizeof(section));
  while (section != END_OF_SECTION) {
    TEFSCUnit* unit = new TEFSCUnit();
    short pIdx = section;
    short x, y;
    infile.read((char*)&x, sizeof(x));
    infile.read((char*)&y, sizeof(y));
    unit->SetPosition(pIdx, x, y);

    String sm = String("Loading Unit: ") + String(index);
    EFSMain->StatusMessage(sm);
    unit->Load(&infile, false, -1, version);

    pos = infile.tellg();
    EFSMain->ShowProgress( 100 * pos / filesize );

    // Add this unit to the Galaxy Unit List:
    Units.push_back(unit);

    GalaxyModule->UnitTable->Append();
    GalaxyModule->UnitTableUnitID->AsInteger = index;
    GalaxyModule->UnitTablePIdx->AsInteger = unit->PIdx();
    GalaxyModule->UnitTableX->AsInteger = unit->X();
    GalaxyModule->UnitTableY->AsInteger = unit->Y();
    GalaxyModule->UnitTableLoyalty->AsInteger = unit->Loyalty();
    GalaxyModule->UnitTableOwner->AsInteger = unit->Owner;
    GalaxyModule->UnitTablePrevOwner->AsInteger = unit->PrevOwner;
    GalaxyModule->UnitTableType->AsInteger = unit->Type;
    GalaxyModule->UnitTabletLvl->AsInteger = unit->tLvl;

    GalaxyModule->UnitTableTTL->AsInteger = unit->Type * 10 + unit->tLvl;

    GalaxyModule->UnitTableMoveType->AsInteger = unit->MoveType;
    GalaxyModule->UnitTableOrders->AsInteger = unit->Orders;
    GalaxyModule->UnitTableExperience->AsInteger = unit->Experience;
    GalaxyModule->UnitTableMovePoints->AsInteger = unit->MovePoints;
    GalaxyModule->UnitTableResRelic->AsInteger = unit->ResRelic.Res;
    GalaxyModule->UnitTableAmount->AsInteger = unit->Amount;
    GalaxyModule->UnitTableHealth->AsInteger = unit->Health;
    GalaxyModule->UnitTableSect->AsInteger = unit->Sect;
    GalaxyModule->UnitTableMoveType->AsInteger = unit->MoveType;
    GalaxyModule->UnitTableUnitNo->AsInteger = unit->UnitNo;
    GalaxyModule->UnitTableFlags->AsInteger = unit->Flags;
    GalaxyModule->UnitTableUsedUnitType->AsInteger = unit->UsedUnitType;
    GalaxyModule->UnitTableUsedUnittLvl->AsInteger = unit->UsedUnittLvl;

    GalaxyModule->UnitTableUUTTL->AsInteger = unit->UsedUnitType * 10 + unit->UsedUnittLvl;

    GalaxyModule->UnitTableCamo->AsInteger = unit->Camo;
    GalaxyModule->UnitTableDestX->AsInteger = unit->DestX;
    GalaxyModule->UnitTableDestY->AsInteger = unit->DestY;
    GalaxyModule->UnitTableMoveCost->AsInteger = unit->MoveCost;
    GalaxyModule->UnitTabletFlags->AsInteger = unit->tFlags;
    GalaxyModule->UnitTableAIOrders->AsInteger = unit->AIOrders;
    GalaxyModule->UnitTableAIData->AsInteger = unit->AIData;
    GalaxyModule->UnitTableAIData2->AsInteger = unit->AIData2;
    GalaxyModule->UnitTableWaitLevel->AsInteger = unit->WaitLevel;

    if (unit->InSpace()) {
      short owner;
      if (unit->Loaned()) owner = unit->PrevOwner;
      else owner = unit->Owner;
      SetStarMapTile(unit->PIdx(), owner, STAR_UNIT);
      unit->SetPosition(unit->PIdx(),
                        GetUnitPositionInSpaceX(unit->PIdx(), unit->Owner),
                        GetUnitPositionInSpaceY(unit->PIdx(), unit->Owner));
    }

    index++;

    infile.read((char*)&section, sizeof(section));

    // Temporary: Save "end of group" marker as a field in the Unit instance:
    if (section == END_OF_GROUP) {
      unit->GroupEndMark = section;
      GalaxyModule->UnitTableGroupEndMark->Value = unit->GroupEndMark;
      infile.read((char*)&section, sizeof(section));
    }

  } // End of Units
  UnitCount = index;

  // Post last record appended to the UnitTable:
  if (UnitCount > 0) {
    GalaxyModule->UnitTable->Post();
    GalaxyModule->UnitTable->FindFirst();
  }

  // Load in Structures:
  EFSMain->StatusMessage("Loading Structures");
  index = 0;
  GalaxyModule->StructureTable->Active = false;
  GalaxyModule->StructureTable->EmptyTable();
  GalaxyModule->StructureTable->Active = true;
  infile.read((char*)&section, sizeof(section));
  while (section != END_OF_SECTION) {
    TEFSCStructure* structure = new TEFSCStructure();
    short pIdx = section;
    short x, y;
    infile.read((char*)&x, sizeof(x));
    infile.read((char*)&y, sizeof(y));
    structure->SetPosition(pIdx, x, y);

    String sm = String("Loading Structure: ") + String(index);
    EFSMain->StatusMessage(sm);
    structure->Load(&infile, false, -1, version);

    pos = infile.tellg();
    EFSMain->ShowProgress( 100 * pos / filesize );

    // Add this unit to the Galaxy Structures List:
    Structures.push_back(structure);

    GalaxyModule->StructureTable->Append();
    GalaxyModule->StructureTableStructID->Value = index;
    GalaxyModule->StructureTablePIdx->Value = structure->PIdx();
    GalaxyModule->StructureTableX->Value = structure->X();
    GalaxyModule->StructureTableY->Value = structure->Y();
    GalaxyModule->StructureTableLoyalty->Value = structure->Loyalty();
    GalaxyModule->StructureTableType->Value = structure->Type;
    GalaxyModule->StructureTableOwner->Value = structure->Owner;
    GalaxyModule->StructureTablePrevOwner->Value = structure->PrevOwner;
    GalaxyModule->StructureTableProdRuinType->Value = structure->ProdRuinType.ProdType;
    GalaxyModule->StructureTableProdInfo->Value = structure->ProdInfo.ProdtLvl;
    GalaxyModule->StructureTableTurnsLeft->Value = structure->TurnsLeft;
    GalaxyModule->StructureTableCityInfo->Value = structure->CityInfo.ResPoints;
    GalaxyModule->StructureTablePrevType->Value = structure->PrevType;
    GalaxyModule->StructureTableUnitLoyalty->Value = structure->UnitLoyalty;
    GalaxyModule->StructureTableStackInfo->Value = structure->StackInfo.UsedUnitType;
    GalaxyModule->StructureTableUsedUnittLvl->Value = structure->UsedUnittLvl;
    GalaxyModule->StructureTableTechType->Value = structure->TechType;
    GalaxyModule->StructureTableHealth->Value = structure->Health;
    GalaxyModule->StructureTableSect->Value = structure->Sect;
    GalaxyModule->StructureTableFlags->Value = structure->Flags;
    GalaxyModule->StructureTableUnitHealth->Value = structure->UnitHealth;
    GalaxyModule->StructureTableTempCount->Value = structure->TempCount;
    GalaxyModule->StructureTableTempCount2->Value = structure->TempCount2;
    GalaxyModule->StructureTableTempCount3->Value = structure->TempCount3;
    GalaxyModule->StructureTableDamage->Value = structure->Damage;

    index++;

    infile.read((char*)&section, sizeof(section));
  } // End of Structures
  StructureCount = index;

  if (StructureCount > 0) {
    GalaxyModule->StructureTable->Post();
    GalaxyModule->StructureTable->FindFirst();
  }

  // Search for owners of each Planet:
  EFSMain->StatusMessage("Finding Planet Owners");
  FindPlanetOwners();

  infile.close();

  Modified = false;
  EFSMain->ShowProgress(0);
  return true;
}
//---------------------------------------------------------------------------

bool TEFSCGalaxy::SaveGalaxyFile(String fileName)
{
  //if (!Modified) return false;

  Update(ProjectAlias);
  
  ofstream outfile(fileName.c_str(), ios::binary);
  if (!outfile) {
    String s = "The specified EFS Classic Galaxy file:\n"
               + fileName + "\ncould not be opened...";
    MessageDlg(s, mtInformation, TMsgDlgButtons() << mbOK, 0);
    return false;
  }

  GALFileName = fileName;

  version = 990000;
  outfile.write((char*)&version, sizeof(version));

  outfile.write((char*)&UnitCount, sizeof(UnitCount));
  outfile.write((char*)&MapTiles, sizeof(MapTiles));

  int index = 0;
  short section;

  EFSMain->StatusMessage("Saving Planets");
  for (int i=0; i<MAX_PLANETS; i++) {
    if (Planets[i]) {
      String sm = String("Saving Planet: ") + String(index);
      EFSMain->StatusMessage(sm);
      outfile.write((char*)&Planets[i]->x, sizeof(Planets[i]->x));
      outfile.write((char*)&Planets[i]->y, sizeof(Planets[i]->y));
      outfile.write((char*)&Planets[i]->CurRotPos, sizeof(Planets[i]->CurRotPos));
      outfile.write((char*)&Planets[i]->UnitPos, sizeof(Planets[i]->UnitPos));
      outfile.write((char*)&Planets[i]->Name, sizeof(Planets[i]->Name));
      outfile.write((char*)&Planets[i]->Owner, sizeof(Planets[i]->Owner));
      outfile.write((char*)&Planets[i]->Sect, sizeof(Planets[i]->Sect));
      outfile.write((char*)&Planets[i]->Flags, sizeof(Planets[i]->Flags));
      outfile.write((char*)&Planets[i]->TileSetType, sizeof(Planets[i]->TileSetType));

      outfile.write((char*)&Planets[i]->HexBuffer, sizeof(Planets[i]->HexBuffer));
    }
    index++;
  }

  section = END_OF_SECTION;
  outfile.write((char*)&section, sizeof(section));

  // Save Jump Gates:
  EFSMain->StatusMessage("Saving Jump Routes");
  index = 0;
  for (int i=0; i<MAX_JUMP_GATES; i++) {
    if (JumpGates[i]) {
      String sm = String("Saving Jump Route: ") + String(index);
      EFSMain->StatusMessage(sm);
      short coord = JumpGates[i]->x1;
      outfile.write((char*)&coord, sizeof(coord));
      coord = JumpGates[i]->y1;
      outfile.write((char*)&coord, sizeof(coord));
      coord = JumpGates[i]->x2;
      outfile.write((char*)&coord, sizeof(coord));
      coord = JumpGates[i]->y2;
      outfile.write((char*)&coord, sizeof(coord));
      outfile.write((char*)&JumpGates[i]->Flags, sizeof(JumpGates[i]->Flags));
    }
    index++;
  }

  section = END_OF_SECTION;
  outfile.write((char*)&section, sizeof(section));

  // Save Units:
  EFSMain->StatusMessage("Saving Units");
  index = 0;
  list<TEFSCUnit*>::iterator unit;
  for (unit = Units.begin(); unit != Units.end(); unit++) {
    String sm = String("Saving Unit: ") + String(index);
    EFSMain->StatusMessage(sm);
    (*unit)->Save(&outfile);
    index++;
  }

  section = END_OF_SECTION;
  outfile.write((char*)&section, sizeof(section));

  // Save Structures:
  EFSMain->StatusMessage("Save Structures");
  index = 0;
  // Iterate through Structures list, writing each to the GAL file:
  list<TEFSCStructure*>::iterator structure;
  for (structure = Structures.begin(); structure != Structures.end(); structure++) {
    String sm = String("Saving Structure: ") + String(index);
    EFSMain->StatusMessage(sm);
    (*structure)->Save(&outfile);
    index++;
  }

  section = END_OF_SECTION;
  outfile.write((char*)&section, sizeof(section));

  outfile.close();
  Modified = false;
  EFSMain->StatusMessage("Ready");
  return true;
}
//---------------------------------------------------------------------------

void TEFSCGalaxy::CloseGalaxyFile()
{
  // Warn user that this instance of TEFSCGalaxy is about to be deleted
  //and allow them to save work in progress...  There's no chance to abort
  //at this stage; they either save or risk loosing work done since the
  //last save:
  if (Modified) {
    String s = "Galaxy object is about to be deleted.\n"
               "Do you wish to save first?";
    int result = MessageDlg(s, mtWarning, TMsgDlgButtons() << mbYes << mbNo, 0);
    if (result == mrYes) SaveGalaxyFile(GALFileName);
  }
}
//---------------------------------------------------------------------------

void TEFSCGalaxy::UpdateJumpGates()
{
  // This function matches the Jump Gate starting and ending locations with
  //planet coordinates; when a planet who's position corresponds to a Jump
  //Gate endpoint the Index of that planet is saved in the corresponding
  //field of the TEFSCJumpGate instance.

  for (int i=0; i<JumpGateCount; i++) {
    if (!JumpGates[i]->Flags) continue;
    for (int j=0; j<PlanetCount; j++) {
      if (Planets[j]->x == JumpGates[i]->x1 && Planets[j]->y == JumpGates[i]->y1)
        JumpGates[i]->Planet1Index = j;
      if (Planets[j]->x == JumpGates[i]->x2 && Planets[j]->y == JumpGates[i]->y2)
        JumpGates[i]->Planet2Index = j;
    }
  }

  // If a JumpGate endpoint does not correspond with a planet position, then
  //make that endpoint the same as the other (valid) endpoint.  If both are
  //invalid, then set both Planet1Index and Planet2Index to the first valid
  //planet index:

  int FirstPlanet = -1;
  for (int i=0; i<PlanetCount; i++) {
    if (Planets[i]) FirstPlanet = i;
  }

  if (FirstPlanet >= 0) {
    for (int i=0; i<JumpGateCount; i++) {
      if ((JumpGates[i]->x1 < 0) && (JumpGates[i]->x2 < 0)) {
        JumpGates[i]->Planet1Index = FirstPlanet;
        JumpGates[i]->x1 = Planets[FirstPlanet]->x;
        JumpGates[i]->y1 = Planets[FirstPlanet]->y;
        JumpGates[i]->Planet2Index = FirstPlanet;
        JumpGates[i]->x2 = Planets[FirstPlanet]->x;
        JumpGates[i]->y2 = Planets[FirstPlanet]->y;
      }
      else if (JumpGates[i]->x1 < 0) {
          JumpGates[i]->Planet1Index = JumpGates[i]->Planet2Index;
          JumpGates[i]->x1 = Planets[JumpGates[i]->Planet2Index]->x;
          JumpGates[i]->y1 = Planets[JumpGates[i]->Planet2Index]->y;
        }
        else if (JumpGates[i]->x2 <0) {
            JumpGates[i]->Planet2Index = JumpGates[i]->Planet1Index;
            JumpGates[i]->x2 = Planets[JumpGates[i]->Planet1Index]->x;
            JumpGates[i]->y2 = Planets[JumpGates[i]->Planet1Index]->y;
          }
    } // End of for loop
  } // End of if FirstPlanet
}

//---------------------------------------------------------------------------
int TEFSCGalaxy::GetVersion()
{
  return version;
}
//---------------------------------------------------------------------------
String TEFSCGalaxy::GetGALFileName()
{
  return GALFileName;
}
//---------------------------------------------------------------------------

void TEFSCGalaxy::Update(String projectAlias)
{
  // Synchronize all fileds of objects and components reflected in the
  //database in preparation for saving to EFS Classic format

  GALFileName = "";
  version = 990000;

  // Set Data Module to point to the correct Alias...
  SetProjectAlias(projectAlias);

  // Activate DataSets as necessary...
  GalaxyModule->ActivateAllTables();

  // Return all tables to unfiltered state and index on ID's...
  GalaxyModule->FilterDataSets(false);
  

  // Flush the MapTiles array:
  for (int i=0; i<STAR_MAP_WIDTH; i++) {
    for (int j=0; j<STAR_MAP_HEIGHT; j++) MapTiles[i][j] = 0;
  }

  // Flush the Planets array:
  for (int i=0; i<MAX_PLANETS; i++) {
    if (Planets[i]) {
      delete Planets[i];
      Planets[i] = 0;
    }
  }

  // Flush the JumpGates array:
  for (int i=0; i<MAX_JUMP_GATES; i++) {
    if (JumpGates[i]) {
      delete JumpGates[i];
      JumpGates[i] = 0;
    }
  }

  // Set up Planets array:
  EFSMain->StatusMessage("Updating Planets");
  EFSMain->ShowProgress(10);

  int index = 0;

  bool PlanetsExist = GalaxyModule->PlanetTable->RecordCount;
  if (PlanetsExist) {
    /* DONE -oMitchell : Specify directory for FS2 map file */
    String PMFile = GalaxyModule->GetDataDirectory() + String("PlanetMaps.FS2");
    ifstream mapfile(PMFile.c_str(), ios::binary);
    if (!mapfile) {
      String s = String("The specified EFS Planet Maps file:\n")
                 + String("PlanetMaps.FS2") + String("\ncould not be opened...");
      MessageDlg(s, mtInformation, TMsgDlgButtons() << mbOK, 0);
      return;
    }
 
    GalaxyModule->PlanetTable->First();
    while (!GalaxyModule->PlanetTable->Eof) {
      index = GalaxyModule->PlanetTablePIdx->Value;
      if (index < MAX_PLANETS) {
        Planets[index] = new TEFSCPlanet(index);
        Planets[index]->SetName(GalaxyModule->PlanetTableName->AsString);
        Planets[index]->x = GalaxyModule->PlanetTableX->Value;
        Planets[index]->y = GalaxyModule->PlanetTableY->Value;
        Planets[index]->Owner = GalaxyModule->PlanetTableOwner->Value;
        Planets[index]->Sect = GalaxyModule->PlanetTableSect->Value;
        Planets[index]->CurRotPos = GalaxyModule->PlanetTableCRP->Value;
        Planets[index]->Flags = GalaxyModule->PlanetTableFlags->Value;
        Planets[index]->TileSetType = GalaxyModule->PlanetTableTileSet->Value;

        mapfile.read((char*)&Planets[index]->HexBuffer, sizeof(Planets[index]->HexBuffer));

        /*WideString wsMapHexes((wchar_t*)Planets[index]->HexBuffer, PLANET_MAP_WIDTH * PLANET_MAP_HEIGHT);
        int wsMHLength = wsMapHexes.Length();

        //strcpy((char*)Planets[index]->HexBuffer, (char*)GalaxyModule->PlanetTableHexBuffer->AsString.data());
        for (int k=0; k<wsMHLength; k++) {
          Planets[index]->HexBuffer[k] = wsMapHexes[k+1];
        }
        */

        SetStarMapTile(index, Planets[index]->Owner, STAR_PLANET);
      }
      GalaxyModule->PlanetTable->Next();
    }
    GalaxyModule->PlanetTable->First();
    mapfile.close();
  }
  
  // Count the number of valid planets:
  PlanetCount = 0;
  for (int i = 0; i<MAX_PLANETS; i++) {
    if (Planets[i]) {
      if (Planets[i]->GetName() != String("")) PlanetCount++;
    }
  }

  // Set up JumpGates array:
  EFSMain->StatusMessage("Updating Jump Routes");
  EFSMain->ShowProgress(20);
  GalaxyModule->JumpRouteTable->First();
  while (!GalaxyModule->JumpRouteTable->Eof) {
    index = GalaxyModule->JumpRouteTableJRIdx->Value;
    if (index < MAX_JUMP_GATES) {
      JumpGates[index] = new TEFSCJumpGate();
      JumpGates[index]->Planet1Index = GalaxyModule->JumpRouteTableP1Idx->Value;
      JumpGates[index]->Planet2Index = GalaxyModule->JumpRouteTableP2Idx->Value;
      if (Planets[JumpGates[index]->Planet1Index]) {
        JumpGates[index]->x1 = Planets[JumpGates[index]->Planet1Index]->x;
        JumpGates[index]->y1 = Planets[JumpGates[index]->Planet1Index]->y;
      }
      if (Planets[JumpGates[index]->Planet2Index]) {
        JumpGates[index]->x2 = Planets[JumpGates[index]->Planet2Index]->x;
        JumpGates[index]->y2 = Planets[JumpGates[index]->Planet2Index]->y;
      }
      JumpGates[index]->Flags = GalaxyModule->JumpRouteTableFlags->Value;
    }
    GalaxyModule->JumpRouteTable->Next();
  }
  GalaxyModule->JumpRouteTable->First();

  // Count the number of valid Jump Routes:
  JumpGateCount = 0;
  for (int i = 0; i<MAX_JUMP_GATES; i++) {
    if (JumpGates[i]) {
      if ((JumpGates[i]->Planet1Index >= 0) && (JumpGates[i]->Planet2Index >= 0)) JumpGateCount++;
    }
  }

  // Flush and re-populate Units list:
  EFSMain->StatusMessage("Updating Units");
  EFSMain->ShowProgress(30);

  UnitCount = GalaxyModule->UnitTable->RecordCount;

  while (Units.size() > 0) {
    delete Units.back();
    Units.pop_back();
  }

  GalaxyModule->UnitTable->First();
  while (!GalaxyModule->UnitTable->Eof) {
    TEFSCUnit* unit = new TEFSCUnit();

    index = GalaxyModule->UnitTableUnitID->Value;
    unit->SetPosition(GalaxyModule->UnitTablePIdx->Value,
                      GalaxyModule->UnitTableX->Value,
                      GalaxyModule->UnitTableY->Value);
    unit->SetLoyalty(GalaxyModule->UnitTableLoyalty->Value);
    unit->Owner = GalaxyModule->UnitTableOwner->Value;
    unit->PrevOwner = GalaxyModule->UnitTablePrevOwner->Value;
    unit->Type = GalaxyModule->UnitTableType->Value;
    unit->tLvl = GalaxyModule->UnitTabletLvl->Value;
    unit->MoveType = GalaxyModule->UnitTableMoveType->Value;
    unit->Orders = GalaxyModule->UnitTableOrders->Value;
    unit->Experience = GalaxyModule->UnitTableExperience->Value;
    unit->MovePoints = GalaxyModule->UnitTableMovePoints->Value;
    unit->ResRelic.Res = GalaxyModule->UnitTableResRelic->Value;
    unit->Amount = GalaxyModule->UnitTableAmount->Value;
    unit->Health = GalaxyModule->UnitTableHealth->Value;
    unit->Sect = GalaxyModule->UnitTableSect->Value;
    unit->MoveType = GalaxyModule->UnitTableMoveType->Value;
    unit->UnitNo = GalaxyModule->UnitTableUnitNo->Value;
    unit->Flags = GalaxyModule->UnitTableFlags->Value;
    unit->UsedUnitType = GalaxyModule->UnitTableUsedUnitType->Value;
    unit->UsedUnittLvl = GalaxyModule->UnitTableUsedUnittLvl->Value;
    unit->Camo = GalaxyModule->UnitTableCamo->Value;
    unit->DestX = GalaxyModule->UnitTableDestX->Value;
    unit->DestY = GalaxyModule->UnitTableDestY->Value;
    unit->MoveCost = GalaxyModule->UnitTableMoveCost->Value;
    unit->tFlags = GalaxyModule->UnitTabletFlags->Value;
    unit->AIOrders = GalaxyModule->UnitTableAIOrders->Value;
    unit->AIData = GalaxyModule->UnitTableAIData->Value;
    unit->AIData2 = GalaxyModule->UnitTableAIData2->Value;
    unit->WaitLevel = GalaxyModule->UnitTableWaitLevel->Value;
    unit->GroupEndMark = GalaxyModule->UnitTableGroupEndMark->Value;

    if (unit->InSpace()) {
      // A Unit in space occupies it's Previous Owner's slot while "Loaned"
      short owner;
      if (unit->Loaned()) owner = unit->PrevOwner;
      else owner = unit->Owner;
      SetStarMapTile(unit->PIdx(), unit->Owner, STAR_UNIT);
      unit->SetPosition(unit->PIdx(),
                        GetUnitPositionInSpaceX(unit->PIdx(), unit->Owner),
                        GetUnitPositionInSpaceY(unit->PIdx(), unit->Owner));
    }

    // Add this unit to the Galaxy Unit List:
    Units.push_back(unit);

    GalaxyModule->UnitTable->Next();
  }
  GalaxyModule->UnitTable->First();

  // Flush and re-populate Structures list:
  EFSMain->StatusMessage("Updating Structures");
  EFSMain->ShowProgress(80);

  StructureCount = GalaxyModule->StructureTable->RecordCount;

  while (Structures.size() > 0) {
    delete Structures.back();
    Structures.pop_back();
  }

  GalaxyModule->StructureTable->First();
  while (!GalaxyModule->StructureTable->Eof) {
    TEFSCStructure* structure = new TEFSCStructure();

    index = GalaxyModule->StructureTableStructID->Value;
    structure->SetPosition(GalaxyModule->StructureTablePIdx->Value,
                      GalaxyModule->StructureTableX->Value,
                      GalaxyModule->StructureTableY->Value);
    structure->SetLoyalty(GalaxyModule->StructureTableLoyalty->Value);
    structure->Type = GalaxyModule->StructureTableType->Value;
    structure->Owner = GalaxyModule->StructureTableOwner->Value;
    structure->PrevOwner = GalaxyModule->StructureTablePrevOwner->Value;
    structure->ProdRuinType.ProdType = GalaxyModule->StructureTableProdRuinType->Value;
    structure->ProdInfo.ProdtLvl = GalaxyModule->StructureTableProdInfo->Value;
    structure->TurnsLeft = GalaxyModule->StructureTableTurnsLeft->Value;
    structure->CityInfo.ResPoints = GalaxyModule->StructureTableCityInfo->Value;
    structure->PrevType = GalaxyModule->StructureTablePrevType->Value;
    structure->UnitLoyalty = GalaxyModule->StructureTableUnitLoyalty->Value;
    structure->StackInfo.UsedUnitType = GalaxyModule->StructureTableStackInfo->Value;
    structure->UsedUnittLvl = GalaxyModule->StructureTableUsedUnittLvl->Value;
    structure->TechType = GalaxyModule->StructureTableTechType->Value;
    structure->Health = GalaxyModule->StructureTableHealth->Value;
    structure->Sect = GalaxyModule->StructureTableSect->Value;
    structure->Flags = GalaxyModule->StructureTableFlags->Value;
    structure->UnitHealth = GalaxyModule->StructureTableUnitHealth->Value;
    structure->TempCount = GalaxyModule->StructureTableTempCount->Value;
    structure->TempCount2 = GalaxyModule->StructureTableTempCount2->Value;
    structure->TempCount3 = GalaxyModule->StructureTableTempCount3->Value;
    structure->Damage = GalaxyModule->StructureTableDamage->Value;

    Structures.push_back(structure);

    GalaxyModule->StructureTable->Next();
  }
  GalaxyModule->StructureTable->First();

  // Search for owners of each Planet:
  EFSMain->StatusMessage("Finding Planet Owners");
  FindPlanetOwners();

  EFSMain->StatusMessage("Ready");
  EFSMain->ShowProgress(0);

  EFSMain->UpdateViews();
}
//---------------------------------------------------------------------------

void TEFSCGalaxy::SetStarMapTile(short pIdx, short owner, short type)
{
  switch (type)
  {
    case STAR_UNIT: SetUnitInSpaceMapTile(pIdx, owner);
    break;
    case STAR_PLANET:
      MapTiles[Planets[pIdx]->x][Planets[pIdx]->y] = STAR_PLANET;
    break;
  }
}
//---------------------------------------------------------------------------

void TEFSCGalaxy::SetUnitInSpaceMapTile(short pIdx, short owner)
{
  MapTiles[GetUnitPositionInSpaceX(pIdx, owner)][GetUnitPositionInSpaceY(pIdx, owner)] = STAR_UNIT;
}
//---------------------------------------------------------------------------

int TEFSCGalaxy::GetUnitPositionInSpaceX(short pIdx, short owner)
{
  int x = Planets[pIdx]->x;
  switch (owner)
  {
    case HOUSE1: if (x > 0) return x - 1;
                 else return 0;
    case HOUSE3: if (x < (STAR_MAP_WIDTH - 1)) return x + 1;
                 else return STAR_MAP_WIDTH - 1;
    case HOUSE4: if (x > 0) return x - 1;
                 else return 0;
    case HOUSE5: if (x < (STAR_MAP_WIDTH - 1)) return x + 1;
                 else return STAR_MAP_WIDTH - 1;
    case LEAGUE: if (x < (STAR_MAP_WIDTH - 1)) return x + 1;
                 else return STAR_MAP_WIDTH - 1;
    case THE_CHURCH: if (x < (STAR_MAP_WIDTH - 1)) return x + 1;
                     else return STAR_MAP_WIDTH - 1;
    case SYMBIOT: if (x > 0) return x - 1;
                  else return 0;
    case VAU: if (x > 0) return x - 1;
              else return 0;
    default: return x;
  }
}
//---------------------------------------------------------------------------

int TEFSCGalaxy::GetUnitPositionInSpaceY(short pIdx, short owner)
{
  int y = Planets[pIdx]->y;
  switch (owner)
  {
    case HOUSE1: if (y > 0) return y - 1;
                 else return 0;
    case HOUSE2: if (y > 0) return y - 1;
                 else return 0;
    case HOUSE3: if (y > 0) return y - 1;
                 else return 0;
    case HOUSE4: return y;
    case HOUSE5: return y;
    default:
      if (y < (STAR_MAP_HEIGHT - 1)) return y + 1;
      else return STAR_MAP_HEIGHT - 1;

  }
}
//---------------------------------------------------------------------------

bool TEFSCGalaxy::SetDATPath(String datPath)
{
  if (DirectoryExists(datPath)) {
    DATPath = datPath;
    return true;
  }
  else return false;
}
//---------------------------------------------------------------------------

String TEFSCGalaxy::GetDATPath()
{
  return DATPath;
}
//---------------------------------------------------------------------------

void TEFSCGalaxy::SetProjectAlias(String aliasName)
{
  ProjectAlias = aliasName;
  GalaxyModule->SetDatabase(ProjectAlias);
}
//---------------------------------------------------------------------------

void TEFSCGalaxy::SetGALFileName(String galFileName)
{
  GALFileName = galFileName;
  GALPathName = ExtractFilePath(GALFileName);
  DATPath = GALPathName + String("Dat\\");
}
//---------------------------------------------------------------------------

String TEFSCGalaxy::GetGALPathName()
{
  return GALPathName;
}
//---------------------------------------------------------------------------

void TEFSCGalaxy::CreateProject(String aliasName)
{
  ProjectAlias = aliasName;
  GalaxyModule->CreateProject(ProjectAlias);
}
//---------------------------------------------------------------------------

void TEFSCGalaxy::FindPlanetOwners()
{
  GalaxyModule->PlanetTable->First();
  while (!GalaxyModule->PlanetTable->Eof) {
    GalaxyModule->PlanetTable->Edit();
    GalaxyModule->PlanetTableOwner->AsInteger = -1;
    GalaxyModule->PlanetTable->Post();
    Planets[GalaxyModule->PlanetTablePIdx->AsInteger]->Owner = -1;
    GalaxyModule->PlanetTable->Next();
  }

  GalaxyModule->StructureTable->First();
  while (!GalaxyModule->StructureTable->Eof) {
    if (GalaxyModule->StructureTableType->AsInteger == 0) {
      GalaxyModule->PlanetTable->FindKey(OPENARRAY(TVarRec, (GalaxyModule->StructureTablePIdx->AsInteger)));
      GalaxyModule->PlanetTable->Edit();
      GalaxyModule->PlanetTableOwner->AsInteger = GalaxyModule->StructureTableOwner->AsInteger;
      GalaxyModule->PlanetTable->Post();
      Planets[GalaxyModule->StructureTablePIdx->AsInteger]->Owner = GalaxyModule->PlanetTableOwner->AsInteger;
    }
    GalaxyModule->StructureTable->Next();
  }
  GalaxyModule->StructureTable->First();
  GalaxyModule->PlanetTable->First();
}
//---------------------------------------------------------------------------
//---------------------------------------------------------------------------
//---------------------------------------------------------------------------

