//---------------------------------------------------------------------------
#ifndef TEFSCGalaxyH
#define TEFSCGalaxyH

#include "TEFSCPlanet.h"
#include "TEFSCJumpGate.h"
#include "TEFSCUnit.h"
#include "TEFSCStructure.h"

#include <vcl/dstring.h>
#include <list>

const int STAR_MAP_WIDTH = 50;
const int STAR_MAP_HEIGHT = 48;

const int MAX_PLANETS = 40;

const short STAR_EMPTY = 0;
const short STAR_UNIT = 1;
const short STAR_PLANET = 2;

const short END_OF_SECTION = -2;
const short END_OF_GROUP = -3;

const short UNIT_OWN_HOUSE1   = 1044;
const short UNIT_OWN_HOUSE2   = 1045;
const short UNIT_OWN_HOUSE3   = 1046;
const short UNIT_OWN_HOUSE4   = 1047;
const short UNIT_OWN_HOUSE5   = 1048;
const short UNIT_OWN_LEAGUE   = 1049;
const short UNIT_OWN_CHURCH   = 1050;
const short UNIT_OWN_SYMBIOT  = 1051;
const short UNIT_OWN_VAU      = 1052;
const short UNIT_OWN_IMPERIAL = 1053;
const short UNIT_OWN_FLEET    = 1054;
const short UNIT_OWN_STIGMATA = 1055;
const short UNIT_OWN_SPY      = 1056;
const short UNIT_OWN_NEUTRAL  = 1057;

const short HOUSE1      = (UNIT_OWN_HOUSE1    - UNIT_OWN_HOUSE1);
const short HOUSE2      = (UNIT_OWN_HOUSE2    - UNIT_OWN_HOUSE1);
const short HOUSE3      = (UNIT_OWN_HOUSE3    - UNIT_OWN_HOUSE1);
const short HOUSE4      = (UNIT_OWN_HOUSE4    - UNIT_OWN_HOUSE1);
const short HOUSE5      = (UNIT_OWN_HOUSE5    - UNIT_OWN_HOUSE1);
const short LEAGUE      = (UNIT_OWN_LEAGUE    - UNIT_OWN_HOUSE1);
const short THE_CHURCH  = (UNIT_OWN_CHURCH    - UNIT_OWN_HOUSE1);
const short SYMBIOT     = (UNIT_OWN_SYMBIOT   - UNIT_OWN_HOUSE1);
const short VAU         = (UNIT_OWN_VAU       - UNIT_OWN_HOUSE1);
const short IMPERIAL    = (UNIT_OWN_IMPERIAL  - UNIT_OWN_HOUSE1);
const short FLEET       = (UNIT_OWN_FLEET     - UNIT_OWN_HOUSE1);
const short STIGMATA    = (UNIT_OWN_STIGMATA  - UNIT_OWN_HOUSE1);
const short THE_SPY     = (UNIT_OWN_SPY       - UNIT_OWN_HOUSE1);
const short NEUTRAL     = (UNIT_OWN_NEUTRAL   - UNIT_OWN_HOUSE1);

class TEFSCGalaxy
{
private:
  String GALFileName;
  String GALPathName;
  String DATPath;
  String ProjectAlias;
  String AliasPath;
  
  int version;                      // Version date of *.GAL file

protected:

public:
  bool Modified;
  int MapTiles[STAR_MAP_WIDTH][STAR_MAP_HEIGHT];
  short PlanetCount;      // Total number of Planets in this Galaxy
  short JumpGateCount;    // Total number of Jump Gates in this Galaxy
  int StructureCount;     // Total number of Structures (cities) in this Galaxy
  int UnitCount;          // Total number of Units in this Galaxy
  int UnitsInFile;        // Number of units specified by the *.GAL file
  TEFSCPlanet* Planets[MAX_PLANETS];
  TEFSCJumpGate* JumpGates[MAX_JUMP_GATES];
  list<TEFSCUnit*> Units;           // Galaxy Units List
  list<TEFSCStructure*> Structures; // Galaxy Structures List

  TEFSCGalaxy();
  ~TEFSCGalaxy();

  bool OpenGalaxyFile(const String fileName, const String projectAlias);
  bool SaveGalaxyFile(const String fileName);
  void CloseGalaxyFile();

  void UpdateJumpGates();
  int GetVersion();
  String GetGALPathName();
  void SetGALFileName(String galFileName);
  String GetGALFileName();
  bool SetDATPath(String datPath);
  void SetProjectAlias(String aliasName);
  void Update(String projectAlias);
  void SetStarMapTile(short pIdx, short owner, short type);
  void SetUnitInSpaceMapTile(short pIdx, short owner);
  int GetUnitPositionInSpaceX(short pIdx, short owner);
  int GetUnitPositionInSpaceY(short pIdx, short owner);
  void CreateProject(String aliasName);
  String GetDATPath();
  void FindPlanetOwners();

};
//---------------------------------------------------------------------------
#endif
