//---------------------------------------------------------------------------
#include <vcl.h>
#pragma hdrstop

#include <Filectrl.hpp>

#include "TEFSMain.h"
#include "TJumpGatesView.h"
#include "TStarMapView.h"
#include "TGalaxyStatsView.h"
#include "TPlanetListView.h"
#include "TUnitListView.h"
#include "TStructureListView.h"
#include "TFactionView.h"
#include "TDATTextEditor.h"
#include "TPlanetTypeView.h"
#include "TSectView.h"
#include "TNewProjectDialog.h"
#include "TOpenProjectDialog.h"
#include "TGalaxyModule.h"
#include "TEFSAboutBox.h"
#include "TTemplateModule.h"
#include "TDeleteProjectDialog.h"
#include "TDATGridView.h"
#include "TEditUnitTypeDialog.h"
//---------------------------------------------------------------------------
#pragma package(smart_init)
#pragma resource "*.dfm"
TEFSMain *EFSMain;
//---------------------------------------------------------------------------
__fastcall TEFSMain::TEFSMain(TComponent* Owner)
  : TForm(Owner)
{
  EFSCGalaxy = NULL;
  Progress = 0;

}
//---------------------------------------------------------------------------

void __fastcall TEFSMain::OnHint(TObject* Sender)
{
  if (EFSMainStatusBar->SimplePanel) EFSMainStatusBar->SimpleText = Application->Hint;
  else EFSMainStatusBar->Panels->Items[HINT_PANEL]->Text = Application->Hint;
}
//---------------------------------------------------------------------------

void __fastcall TEFSMain::FormCreate(TObject *Sender)
{
  Application->OnHint = &OnHint;
  LastEFSCGalaxyPathName = ExtractFilePath(Application->ExeName) + String("Data\\");
  LastDATPath = LastEFSCGalaxyPathName + String("Dat\\");
  LastEFSCGalaxyFileName = LastEFSCGalaxyPathName + String("Galaxy.gal");

  TemplateAliasName = String("EFS Basic");

}
//---------------------------------------------------------------------------

void __fastcall TEFSMain::FileNewActionExecute(TObject *Sender)
{
  int result = NewProjectDialog->ShowModal();

  if (result != mrCancel) {
    if (EFSCGalaxy) {
      EFSCGalaxy->CloseGalaxyFile();
      delete EFSCGalaxy;
      EFSCGalaxy = NULL;
      FileCloseAllActionExecute(this);
    }
    CurrentAliasName = NewProjectDialog->eNewProjectName->Text;
    EFSCGalaxy = new TEFSCGalaxy();

    switch (result)
    {
      case mrYes: {
        // Start by opening an existing *.GAL file
        OpenEFSCGALDlg->FileName = LastEFSCGalaxyFileName;
        if (OpenEFSCGALDlg->Execute()) {
          if (FileExists(OpenEFSCGALDlg->FileName)) {
            LastEFSCGalaxyFileName = OpenEFSCGALDlg->FileName;
            LastEFSCGalaxyPathName = ExtractFilePath(LastEFSCGalaxyFileName);
            EFSCGalaxy->OpenGalaxyFile(LastEFSCGalaxyFileName, CurrentAliasName);
            LastDATPath = EFSCGalaxy->GetDATPath();
            TemplateAliasName = String("EFS Basic");
            TemplateModule->CopyBasicTables(TemplateAliasName);
            GalaxyModule->ActivateAllTables();
            StatusMessage("Ready");
          }
        }
      }
      break;
      case mrOk: {
        // Start using an existing database as a Template

        // Set up the new database and galaxy object:
        EFSCGalaxy->CreateProject(CurrentAliasName);  // Also calls GalaxyModule->SetDatabase()
        LastEFSCGalaxyPathName = GalaxyModule->GetDataDirectory();
        LastEFSCGalaxyFileName = LastEFSCGalaxyPathName + CurrentAliasName + String(".GAL");
        EFSCGalaxy->SetGALFileName(LastEFSCGalaxyFileName);

        // Set TemplateModule to point to the correct database alias:
        TemplateAliasName = NewProjectDialog->cbAliases->Text;

        // Copy data from template database to the new project database:
        TemplateModule->CopyAllTables(TemplateAliasName);

        // Copy the template planet maps file to the new project directory
        //(overwrite if file exists):
        String TemplateMapFileName = TemplateModule->GetDataDirectory() + String("PlanetMaps.FS2");
        String ProjectMapFileName = GalaxyModule->GetDataDirectory() + String("PlanetMaps.FS2");
        CopyFile(TemplateMapFileName.c_str(), ProjectMapFileName.c_str(), false);

        // Initialize Planet, JumpGate, Unit and Structure objects as specified
        //by the new project database:
        EFSCGalaxy->Update(CurrentAliasName);
        
      }
      break;
      case mrNo: { // Start from scratch

        // Set up the new database and galaxy object:
        EFSCGalaxy->CreateProject(CurrentAliasName);  // Also calls GalaxyModule->SetDatabase()
        LastEFSCGalaxyPathName = GalaxyModule->GetDataDirectory();
        LastEFSCGalaxyFileName = LastEFSCGalaxyPathName + CurrentAliasName + String(".GAL");
        EFSCGalaxy->SetGALFileName(LastEFSCGalaxyFileName);

        // Set TemplateModule to point to the correct database alias:
        TemplateAliasName = ExtractFilePath(Application->ExeName) + String("Data\\");

        // Copy data from template database to the new project database:
        TemplateModule->CopyBasicTables(TemplateAliasName);

        // Save a blank planet maps file to the new project directory
        //(overwrite if file exists):
        String ProjectMapFileName = GalaxyModule->GetDataDirectory() + String("PlanetMaps.FS2");
        /* TODO -oMitchell : Devise a means of creating a blank PlanetMaps file */

        // Initialize Planet, JumpGate, Unit and Structure objects as specified
        //by the new project database:
        EFSCGalaxy->Update(CurrentAliasName);
        
      }
      break;
    }

    ViewStarMapActionExecute(this);
    ViewGalaxyStatsActionExecute(this);
    Caption = String("EFS Editor: ") + CurrentAliasName;
  }

}

//---------------------------------------------------------------------------
void __fastcall TEFSMain::FileExportActionExecute(TObject *Sender)
{
  if (EFSCGalaxy) {
    // Save the *.GAL file:
    SaveEFSCGALDlg->FileName = LastEFSCGalaxyFileName;
    if (SaveEFSCGALDlg->Execute()) {
      LastEFSCGalaxyFileName = SaveEFSCGALDlg->FileName;
      LastEFSCGalaxyPathName = ExtractFilePath(LastEFSCGalaxyFileName);
      LastDATPath = LastEFSCGalaxyPathName + String("Dat\\");
      if (!DirectoryExists(LastDATPath)) {
        CreateDir(LastDATPath);
      }
      EFSCGalaxy->SaveGalaxyFile(LastEFSCGalaxyFileName);

      //Save the *.DAT & EFS.INI files:
      GalaxyModule->SaveDATFiles(LastDATPath);
    }
  }
}
//---------------------------------------------------------------------------

void __fastcall TEFSMain::ViewJumpGateListActionExecute(TObject *Sender)
{
  if (EFSCGalaxy) {
    if (JumpGatesView) JumpGatesView->BringToFront();
    else if (EFSCGalaxy->JumpGateCount > 0) JumpGatesView = new TJumpGatesView(this, EFSCGalaxy);
  }
}
//---------------------------------------------------------------------------

void __fastcall TEFSMain::ViewStarMapActionExecute(TObject *Sender)
{
  if (EFSCGalaxy) {
    if (StarMapView) StarMapView->BringToFront();
    else if (EFSCGalaxy->PlanetCount > 0) StarMapView = new TStarMapView(this, EFSCGalaxy);
  }
}
//---------------------------------------------------------------------------

void __fastcall TEFSMain::EFSMainStatusBarDrawPanel(TStatusBar *StatusBar,
      TStatusPanel *Panel, const TRect &Rect)
{
  TColor SaveColor = StatusBar->Canvas->Brush->Color;
  StatusBar->Canvas->Brush->Color = clBtnFace;
  StatusBar->Canvas->FillRect(Rect);
  TRect ProgressRect;
  ProgressRect.left = Rect.left-1;
  ProgressRect.top = Rect.top+1;
  ProgressRect.bottom = Rect.bottom-1;
  ProgressRect.right = Rect.left + (Rect.right - Rect.left) * Progress / 100 - 1;
  StatusBar->Canvas->Brush->Color = clBlue;
  StatusBar->Canvas->FillRect(ProgressRect);
  StatusBar->Canvas->Brush->Color = SaveColor;

  /*
  // Plot text within progress bar; however, DrawText also paints the
  //background using the current Brush color!
  TPenMode SavePenMode = StatusBar->Canvas->Pen->Mode;
  StatusBar->Canvas->Pen->Mode = pmXor;
  // Must cast Rect to be non-const otherwise a type mismatch results (?)...
  DrawText(StatusBar->Canvas->Handle, StatusBar->Panels->Items[ID_PROGRESS_PANEL]->Text.c_str(),
           -1, &(const_cast<TRect&>(Rect)), DT_SINGLELINE | DT_CENTER);
  StatusBar->Canvas->Pen->Mode = SavePenMode;
  */

}
//---------------------------------------------------------------------------

void TEFSMain::StatusMessage(String message)
{
  EFSMainStatusBar->Panels->Items[STATUS_PANEL]->Text = message;
}
//---------------------------------------------------------------------------

void TEFSMain::ShowProgress(int progress)
{
  if (progress < 0) progress = 0;
  if (progress > 100) progress = 100;
  Progress = progress;
  EFSMainStatusBar->Repaint();
}
//---------------------------------------------------------------------------

void __fastcall TEFSMain::ViewGalaxyStatsActionExecute(TObject *Sender)
{
  if (EFSCGalaxy) {
    GalaxyStatsView = new TGalaxyStatsView(this, EFSCGalaxy);
    GalaxyStatsView->ShowModal();
    delete GalaxyStatsView;
  }
}
//---------------------------------------------------------------------------

void __fastcall TEFSMain::ViewPlanetListActionExecute(TObject *Sender)
{
  if (EFSCGalaxy) {
    if (PlanetListView) PlanetListView->BringToFront();
    else PlanetListView = new TPlanetListView(this);
  }

}
//---------------------------------------------------------------------------

void __fastcall TEFSMain::ViewUnitListActionExecute(TObject *Sender)
{
  if (EFSCGalaxy) {
    if (UnitListView) UnitListView->BringToFront();
    else UnitListView = new TUnitListView(this);
  }

}
//---------------------------------------------------------------------------

void __fastcall TEFSMain::ViewStructureListActionExecute(TObject *Sender)
{
  if (EFSCGalaxy) {
    if (StructureListView) StructureListView->BringToFront();
    else StructureListView = new TStructureListView(this);
  }

}
//---------------------------------------------------------------------------

void __fastcall TEFSMain::DataFactionsActionExecute(TObject *Sender)
{
  if (EFSCGalaxy) {
    if (FactionView) FactionView->BringToFront();
    else FactionView = new TFactionView(this);
  }

}
//---------------------------------------------------------------------------

void __fastcall TEFSMain::DataSectsActionExecute(TObject *Sender)
{
  if (EFSCGalaxy) {
    if (SectView) SectView->BringToFront();
    else SectView = new TSectView(this);
  }

}
//---------------------------------------------------------------------------

void __fastcall TEFSMain::DataPlanetTypeActionExecute(TObject *Sender)
{
  if (EFSCGalaxy) {
    if (PlanetTypeView) PlanetTypeView->BringToFront();
    else PlanetTypeView = new TPlanetTypeView(this);
  }

}
//---------------------------------------------------------------------------

void __fastcall TEFSMain::EditDATText(TObject *Sender)
{
  if (EFSCGalaxy) {
    /* DONE -oMitchell : Close DBGridView window if one exists */
    if (DATGridView) DATGridView->Close();
    
    String FileName;
    TAction* A = dynamic_cast<TAction*>(Sender);
    if (A) FileName = LastDATPath + A->Caption;
    else return;

    if (DATTextEditor) {
      DATTextEditor->Editor->Clear();
      DATTextEditor->BringToFront();
    }
    else {
      DATTextEditor = new TDATTextEditor(this);

    }
    DATTextEditor->LoadFromFile(FileName);
  }
}
//---------------------------------------------------------------------------

void __fastcall TEFSMain::FileOpenActionExecute(TObject *Sender)
{
  int result = OpenProjectDialog->ShowModal();

  if (result == mrOk) {
    if (EFSCGalaxy) {
      EFSCGalaxy->CloseGalaxyFile();
      delete EFSCGalaxy;
      EFSCGalaxy = NULL;
      FileCloseAllActionExecute(this);
    }
    EFSCGalaxy = new TEFSCGalaxy();

    CurrentAliasName = OpenProjectDialog->cbAliases->Text;
    EFSCGalaxy->Update(CurrentAliasName);
    StatusMessage("Ready");

    ViewStarMapActionExecute(this);
    ViewGalaxyStatsActionExecute(this);
    Caption = String("EFS Editor: ") + CurrentAliasName;
  }
}
//---------------------------------------------------------------------------

void TEFSMain::UpdateViews()
{
  if (StarMapView) StarMapView->UpdateView();

}
//---------------------------------------------------------------------------

void __fastcall TEFSMain::ImportDATFileSetActionExecute(TObject *Sender)
{
  // First, querey the user for the directory path containing the set of DAT
  //files to be imported to the current project:
  String ImportDATPath("");
  if (DirectoryExists(LastDATPath)) {
    ImportDATPath = LastDATPath;
  }
  else {
    ImportDATPath = GetCurrentDir();
  }

  // Copy the specified DAT file set to the Project \DAT directory (?):
  // NO: The (possibly edited) DAT files will be saved appropriately
  //when the user later exports the database to the GAL/DAT/INI format.

  // Direct the GalaxyModule to load the text DAT files to the current
  //Project database:
  
  if (SelectDirectory(ImportDATPath, TSelectDirOpts(), 0)) {
    ImportDATPath = IncludeTrailingBackslash(ImportDATPath);
    if (EFSCGalaxy) EFSCGalaxy->SetDATPath(ImportDATPath);
    GalaxyModule->LoadDATFiles(ImportDATPath);
    LastDATPath = ImportDATPath;
  }
  else {
    String s = "No EFS Classic DAT files found in specified path:\n"
               + ImportDATPath;
    MessageDlg(s, mtInformation, TMsgDlgButtons() << mbOK, 0);
  }
  
}
//---------------------------------------------------------------------------

void __fastcall TEFSMain::HelpAboutActionExecute(TObject *Sender)
{
  EFSAboutBox->ShowModal();  
}
//---------------------------------------------------------------------------

void __fastcall TEFSMain::FileCloseAllActionExecute(TObject *Sender)
{
  int nChildren = MDIChildCount - 1;
  for (int i=nChildren; i>=0; i--) {
    MDIChildren[i]->Close();
  }
  if (EFSCGalaxy) {
    EFSCGalaxy->CloseGalaxyFile();
    delete EFSCGalaxy;
    EFSCGalaxy = NULL;
  }
  Caption = String("EFS Editor");
}
//---------------------------------------------------------------------------

void __fastcall TEFSMain::DeleteProjectActionExecute(TObject *Sender)
{
  DeleteProjectDialog->ShowModal();  
}
//---------------------------------------------------------------------------

void __fastcall TEFSMain::DATGridViewActionExecute(TObject *Sender)
{
  if (EFSCGalaxy) {
    /* DONE -oMitchell : Close DATTextEditor window if one exists */
    if (DATTextEditor) DATTextEditor->Close();
    
    if (DATGridView) DATGridView->BringToFront();
    else DATGridView = new TDATGridView(this);
  }

}
//---------------------------------------------------------------------------

void __fastcall TEFSMain::ToolsMenuClick(TObject *Sender)
{
  if (UnitListView) ExchangeUnitTypesAction->Enabled = true;
  else ExchangeUnitTypesAction->Enabled = false;
}
//---------------------------------------------------------------------------

void __fastcall TEFSMain::ExchangeUnitTypesActionExecute(TObject *Sender)
{
  EditUnitTypeDialog->ShowModal();
}
//---------------------------------------------------------------------------

