//---------------------------------------------------------------------------

#include <vcl.h>
#pragma hdrstop

#include "TNewProjectDialog.h"
#include "TEFSMain.h"
//---------------------------------------------------------------------------
#pragma package(smart_init)
#pragma resource "*.dfm"
TNewProjectDialog *NewProjectDialog;
//---------------------------------------------------------------------------
__fastcall TNewProjectDialog::TNewProjectDialog(TComponent* Owner)
  : TForm(Owner)
{
}
//---------------------------------------------------------------------------
void __fastcall TNewProjectDialog::FormShow(TObject *Sender)
{
  NewProjectDirCreated = false;
  TStringList* AliasNames = new TStringList();
  Session->GetAliasNames(AliasNames);
  cbAliases->Items->Clear();
  for (int i=0; i<AliasNames->Count; i++) {
    if (Session->GetAliasDriverName(AliasNames->Strings[i]) == String("STANDARD")) {
      TStringList* TableNames = new TStringList();
      String Pattern = "";
      Session->GetTableNames(AliasNames->Strings[i], Pattern, true, false, TableNames);
      bool IsEFSEditorAlias = false;
      for (int i=0; i<TableNames->Count; i++) {
        if (TableNames->Strings[i].UpperCase() == String("JumpRoutes.db").UpperCase()) {
          IsEFSEditorAlias = true;
        }
      }
      if (IsEFSEditorAlias) cbAliases->Items->Add(AliasNames->Strings[i]);
      delete TableNames;
    }
  }
  delete AliasNames;
  cbAliases->ItemIndex = 0;
  cbAliasesChange(Sender);
  lbNPDirectory->Directory = ExtractFilePath(Application->ExeName) + String("Projects\\");
}
//---------------------------------------------------------------------------
void __fastcall TNewProjectDialog::cbAliasesChange(TObject *Sender)
{
  TStringList* Params = new TStringList();
  Session->GetAliasParams(cbAliases->Text, Params);
  String WD = Params->Strings[0];
  WD.Delete(1, 5);
  WD = IncludeTrailingBackslash(WD);
  stWorkingDirectory->Caption = WD;
  delete Params;
}
//---------------------------------------------------------------------------
void __fastcall TNewProjectDialog::FormCloseQuery(TObject *Sender,
      bool &CanClose)
{
  if (ModalResult != mrCancel) {
    String NPName = eNewProjectName->Text;
    String WDName = IncludeTrailingBackslash(lNPDirectory->Caption);
    // Check to see that the entered "New Project Name" is valid:
    if (NPName == "" || !DirectoryExists(WDName)) {
      String s = "You must enter a Project Name and\n"
                 "create or select a Working Directory";
      MessageDlg(s, mtError, TMsgDlgButtons() << mbOK, 0);
      CanClose = false;
      return;
    }

    // Check to see whether the entered "New Project Name" already exists
    //as an alias on the system:
    TStringList* AliasNames = new TStringList();
    Session->GetAliasNames(AliasNames);
    bool AliasExists = false;
    for (int i=0; i<AliasNames->Count; i++) {
      if (AliasNames->Strings[i] == NPName)
        AliasExists = true;
    }
    delete AliasNames;

    if (AliasExists) {
      // Test whether the existing alias points to an EFS Editor project:
      TStringList* TableNames = new TStringList();
      String Pattern = "";
      Session->GetTableNames(NPName, Pattern, true, false, TableNames);
      bool IsEFSEditorAlias = false;
      for (int i=0; i<TableNames->Count; i++) {
        if (TableNames->Strings[i].UpperCase() == String("JumpRoutes.db").UpperCase()) {
          IsEFSEditorAlias = true;
        }
      }
      delete TableNames;
      if (IsEFSEditorAlias) {
        // Change the Working Directory to the one pointed to by the existing
        //alias:
        TStringList* Params = new TStringList();
        Session->GetAliasParams(NPName, Params);
        WDName = Params->Strings[0];
        delete Params;
        WDName.Delete(1, 5);
        WDName = IncludeTrailingBackslash(WDName);
        char drive = WDName[1];
        if (drive != cbNPDrive->Drive) cbNPDrive->Drive = drive;
        lbNPDirectory->Directory = WDName;

        String s = String("The chosen Project Name, ") + NPName +
                   String(", already exists.\nDo you wish to overwrite this project?");
        int result = MessageDlg(s, mtConfirmation	, TMsgDlgButtons() << mbYes << mbNo, 0);
        if (result == mrYes) {
          // DeleteFile apparently does not accept wildcard characters;
          //it may not be necessary to actually DELETE all of the files
          //anyway, as long as the later routines that set up the new
          //project files REPLACE all of the existing files...
          
          //DeleteFile(WDName + "*.*");
        }
        else {
          CanClose = false;
          return;
        }
      }
      else {
        String s = "The chosen Project Name, " + NPName +
                   "\nis already in use by another application;"
                   "\nplease enter a different Project Name...";
        MessageDlg(s, mtError, TMsgDlgButtons() << mbOK, 0);
        CanClose = false;
        return;
      }
      // No need to set up a new alias since the user chose the name of an existing one
      //and agreed to overwrite it, so just return:
      CanClose = true;
      return;
    }

    // Everything looks OK, so create the requested alias pointing to the selected
    //Working Directory and save the BDE "Config" file:
    Session->AddStandardAlias(NPName, WDName, "Paradox");

    // Test whether the new alias points to an existing EFS Editor project:
    TStringList* TableNames = new TStringList();
    String Pattern = "";
    Session->GetTableNames(NPName, Pattern, true, false, TableNames);
    bool IsEFSEditorAlias = false;
    for (int i=0; i<TableNames->Count; i++) {
      if (TableNames->Strings[i].UpperCase() == String("JumpRoutes.db").UpperCase()) {
        IsEFSEditorAlias = true;
      }
    }
    delete TableNames;
    if (IsEFSEditorAlias) {
      String s = "An existing EFS Project already resides in"
                 "\nspecified folder;"
                 "\nYou should create a new folder in which to"
                 "\nmaintain this new EFS Editor Project...";
      MessageDlg(s, mtWarning, TMsgDlgButtons() << mbOK, 0);
      NewDirectoryButtonClick(this);
      Session->DeleteAlias(NPName);
      WDName = IncludeTrailingBackslash(lNPDirectory->Caption);
      Session->AddStandardAlias(NPName, WDName, "Paradox");
    }

    Session->SaveConfigFile();
    CanClose = true;
  }
  else CanClose = true; // The user "Canceled" the dialog, so let it close...
}
//---------------------------------------------------------------------------

void __fastcall TNewProjectDialog::NewDirectoryButtonClick(TObject *Sender)
{
  String WDName = IncludeTrailingBackslash(lNPDirectory->Caption);
  SelectDirectory(WDName, TSelectDirOpts() << sdAllowCreate << sdPerformCreate << sdPrompt, 0);
  lbNPDirectory->Directory = WDName;
}
//---------------------------------------------------------------------------


